# -*- coding: utf-8 -*-
# Copyright (C) 2014-2024  EDF
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See https://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#
# Author : Konstantin Leontev (OpenCascade S.A.S)

import os
import sys
import importlib.util
from traceback import format_exc

from qtsalome import QApplication, QFileDialog, QMessageBox

from salome.kernel.services import ObjectToID
from salome.geom.geomtools import GeomStudyTools

from .geomrepairadv_progress import RepairProgressDialog
from .geomrepairadv_logger import logger


# Testing
import salome
from salome.geom import geomBuilder


def module_from_filename(filename):
    """
    Create and execute a module by filename.

    Args:
        filename - a given python filename.

    Returns:
        Module.
    """

    # Get the module from the filename
    basename = os.path.basename(filename)
    module_name, _ = os.path.splitext(basename)

    spec = importlib.util.spec_from_file_location(module_name, filename)
    if not spec:
        logger.error('Could not get a spec for %s file!', filename)
        return None

    module = importlib.util.module_from_spec(spec)
    if not module:
        logger.error('Could not get a module for %s file!', filename)
        return None

    sys.modules[module_name] = module

    if not spec.loader:
        logger.error('spec.loader is None for %s module!', module_name)
        return None

    try:
        spec.loader.exec_module(module)

    except FileNotFoundError:
        logger.error(format_exc())
        QMessageBox.warning(
                None,
                'Error',
                'Cannot find module {}!\nOperation was canceled.'.format(module_name)
                )

        return None

    return module


def execute(selected_object, algo_name, args_dict, is_dump_on = True, is_copy_on = True):
    """
    Executes GEOM advanced repair algorithm.

    Args:
        selected_object - geom object selected by user for algorithm
        algo_name - path to the algo module
        args_dict - dictionary with arguments those are specific for each algo
        is_dump_on - if True saves the call to the Python dump
        is_copy_on - if True makes copy of selected_object to pass into algo script.

    Returns:
        Result GEOM object or None if failed or canceled.
    """

    logger.debug('execute() start')

    # Find a module to execute
    algo_module = module_from_filename(algo_name)
    logger.debug('algo_module: %s', algo_module)
    if not algo_module:
        return None

    # Keep the args for python dump
    args_dict_str = str(args_dict)
    logger.debug('args_dict_str: {}'.format(args_dict_str))

    # Make copy to prevent unintentional changing of a source object from the algo script
    geompy = geomBuilder.New()
    if is_copy_on:
        selected_copy = geompy.MakeCopy(
            selected_object, args_dict['result_name'] + '_temp')
    else:
        selected_copy = selected_object

    # Add the copy object as a source
    args_dict['source_solid'] = selected_copy

    logger.debug('Create RepairProgressDialog...')
    progress_dlg = RepairProgressDialog(parent=None, target=algo_module.run, args=args_dict)
    progress_dlg.exec()

    # Delete a copy object if we really have one
    if is_copy_on:
        copy_entry = ObjectToID(selected_copy)
        tools = GeomStudyTools()
        tools.deleteShape(copy_entry)

    # Python dump if execution was completed without errors
    if progress_dlg.is_completed():
        result_object = progress_dlg.get_result()

        # Completed execution doesn't guarantee that we received a valid object
        if not result_object:
            logger.error('Could not get a result object after exec of %s file!', str(algo_name))
            return None

        if is_dump_on:
            geompy.FuncToPythonDump(
                selected_object,
                result_object,
                'from salome.geom.geomrepairadv import geomrepairadv_execute\n',
                'geomrepairadv_execute.execute',
                '\'' + str(algo_name) + '\', ' + args_dict_str
            )

        return result_object

    return None


def test_execution():
    """
    Tests execution of repair algo script.
    It uses PartitionCube.brep file to run merge_faces algorithm.

    Because of relative import must be run from a parent dir as a module:
    $ python -m RepairGUIAdv.geomrepairadv_execute
    """

    salome.salome_init()
    geompy = geomBuilder.New()

    cube_file, _ = QFileDialog.getOpenFileName(None, 'Open brep', '/home', 'Brep Files (*.brep)')
    if not cube_file:
        return

    # cube_file = "PartitionCube.brep"
    source_solid = geompy.ImportBREP(cube_file)

    # Récupération des faces à fusionner
    face_a = geompy.GetFaceNearPoint(source_solid, geompy.MakeVertex(-143, -127, 250))
    face_b = geompy.GetFaceNearPoint(source_solid, geompy.MakeVertex(49,-127,250))
    faces_ids = geompy.GetSubShapesIDs(source_solid, [face_a, face_b])

    geompy.addToStudy(source_solid, "source_solid")
    geompy.addToStudyInFather(source_solid, face_a, "face_a")
    geompy.addToStudyInFather(source_solid, face_b, "face_b")


    args_dict = {
            'source_solid': source_solid,
            'faces_ids': faces_ids,
            'result_name': 'MergeFaces_result'
    }

    current_dir = os.path.dirname(os.path.realpath(__file__))
    algo_filename, _ = QFileDialog.getOpenFileName(
        None, 'Open alogrithm script', current_dir, 'Python Files (*.py)')
    if not algo_filename:
        return

    execute(source_solid, algo_filename, args_dict)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    test_execution()
    sys.exit(app.exec_())
