# -*- coding: utf-8 -*-
# Copyright (C) 2014-2024  EDF
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See https://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#
# Author : Konstantin Leontev (OpenCascade S.A.S)

import sys
import logging
from time import sleep

from qtsalome import QApplication, QPlainTextEdit, \
    QDialog, QVBoxLayout, QProgressDialog, QPushButton

from .geomrepairadv_logger import QTextEditLogger
from .geomrepairadv_worker import Worker


class RepairProgressDialog(QDialog, QPlainTextEdit):
    """
    Dialog to show progress bar and log window during of execution
    of a shape repair script.
    """

    def __init__(self, parent=None, target=None, args=None):
        """
        Inits progress dialogs widgets and runs a target function in a thread.

        Args:
            parent - parent dialog
            target - target function to run
            args - arguments to pass into a target.
        """

        super().__init__(parent)

        self.setWindowTitle('Shape Repair')

        # QProgressDialog uses manual layout, then it's not easy to customize it.
        # So, we use it as a widget that is embedded into the dialog.
        self.progress = QProgressDialog()
        self.progress.setLabelText('Operation in progress...')
        self.progress.setAutoClose(False)
        self.progress.setAutoReset(False)

        # Override default cancel slot to prevent progress from hiding
        self.cancel_button = self.progress.findChild(QPushButton)
        self.cancel_button.clicked.disconnect(self.progress.canceled)
        self.cancel_button.clicked.connect(self.cancel)

        # Helper flag to decide if we need to change button or close the dialog
        self.canceled = False

        # Helper flag to check if execution was completed without errors
        self.completed = False

        # Set logger to redirect logs output into the text widget
        self.log_handler = QTextEditLogger(self)
        logging.getLogger().addHandler(self.log_handler)
        logging.getLogger().setLevel(logging.DEBUG)

        # Layout widgets
        layout = QVBoxLayout(self)

        layout.addWidget(self.log_handler.text_widget)
        layout.addWidget(self.progress)

        self.setLayout(layout)

        # Setup and run target function in a working thread
        self.thread = Worker(parent=self, target=target, args=args)
        self.thread.start()


    def cancel(self):
        """
        Replicates QProgressDialog.cancel() method.
        """

        # We need to keep the dialog opened on the first click
        # so we can see the log output.
        # After that we can close it with a second click.
        if self.canceled:
            self.close()
        else:
            # Terminates the execution of the thread.
            # TODO: find out if we can do it with requestInterruption()
            self.thread.terminate()

            self.progress.setLabelText('Canceled!')
            self.progress.setCancelButtonText('Close')

            # Next click we exit
            self.canceled = True


    def on_failed(self):
        """
        Decided what to do if opreation failed.
        """

        self.progress.setLabelText('Operation failed!')
        self.progress.setCancelButtonText('Close')

        self.canceled = True


    def on_completed(self):
        """
        Decided what to do when opreation completed.
        """

        self.progress.setLabelText('Completed!')
        self.progress.setCancelButtonText('Close')

        # Set the Close button to actually close dialog
        self.canceled = True

        # Lets us know that we get the job done
        self.completed = True


    def value(self):
        """
        Replicates QProgressDialog.value() method.
        """

        return self.progress.value()


    def setValue(self, progress):
        """
        Replicates QProgressDialog.setValue() method.

        Args:
            progress - a new value for a progress bar.
        """

        return self.progress.setValue(progress)


    def close(self):
        """
        Process a close event to remove a log handler.
        """

        logging.getLogger().removeHandler(self.log_handler)

        super().close()


    def is_completed(self):
        """
        Returns True if execution was completed without errors and wasn't canceled.
        """

        return self.completed


    def get_result(self):
        """
        Returns result of the execution or None if the execution failed.
        """

        return self.thread.get_result()


def test_thread():
    """
    Tests running a test function in a thread while
    show a progress with RepairProgressDialog dialog.

    Because of relative import must be run from a parent dir as a module:
    $ python -m RepairGUIAdv.geomrepairadv_progress
    """

    progress_dlg = RepairProgressDialog(parent=None, target=test, args=None)
    progress_dlg.exec()

    if progress_dlg.is_completed():
        logging.info('result: %s', progress_dlg.get_result())
    else:
        logging.info('Cannot get results because execution was not completed.')


def test(args, progress_emitter):
    """
    Tests logging and progress update with RepairProgressDialog.
    """

    if args:
        pass

    progress_emitter.emit()
    logging.debug('debug msg')

    sleep(2)
    # raise Exception

    progress_emitter.emit()
    logging.info('info msg')

    sleep(2)

    progress_emitter.emit()
    logging.warning('warning msg')

    sleep(2)

    logging.error('error msg')
    progress_emitter.emit()

    sleep(2)

    progress_emitter.emit()

    return "Result from test!"


if __name__ == '__main__':
    app = QApplication(sys.argv)
    test_thread()
    sys.exit(app.exec_())
