# -*- coding: utf-8 -*-
# Copyright (C) 2014-2024  EDF
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See https://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#
# Author : Duc Anh HOANG (OpenCascade S.A.S)

import sys

from qtsalome import QGridLayout, QFrame, QApplication, \
    QComboBox, QLabel, QPushButton, QMessageBox, QCheckBox

from salome.geom.geomrepairadv.basedlg import BaseDlg
from salome.geom import geomBuilder
from libGEOM_Swig import GEOM_Swig

from .geomrepairadv_common import DlgRef_1Spin_QTD
from .geomrepairadv_execute import execute
from .geomrepairadv_logger import logger
import GEOM

class Reparation_plugin(BaseDlg):
    """
    Dialog for testing.
    """
    def __init__(self, selection_level = GEOM.EDGE):

        # Implement widget's content here
        main_widget = QFrame()
        layout = QGridLayout(main_widget)
        layout.setContentsMargins(0, 0, 0, 0)

        # Min/max values widgets
        decimals = 2
        max_value = sys.float_info.max
        self._min_widget = DlgRef_1Spin_QTD('Tol Min', 0, decimals, max_value)
        self._max_widget = DlgRef_1Spin_QTD('Tol Max', 1000, decimals, max_value)
        self._min_widget.SpinBox_DX.valueChanged.connect(self.on_limit_changed)
        self._max_widget.SpinBox_DX.valueChanged.connect(self.on_limit_changed)

        # Debug checkbox
        #_debug_label = QLabel('Debug')
        self._debug = QCheckBox("Debug")
        self._debug.setChecked(True)
        self._debug.stateChanged.connect(lambda:self.onDebugStatechanged())

        # Add the widgets to layout
        layout.addWidget(self._min_widget, 1, 0)
        layout.addWidget(self._max_widget, 2, 0)
        layout.addWidget(self._debug, 3, 0)


        # Init base dialog
        BaseDlg.__init__(
            self,
            main_widget,
            'Reparation plugin',
            'reparation_plugin_algo.py',
            True,
            selection_level
            )

        # Adjust setup from a base class
        self._sel_subshape_widget.hide()
        self._is_copy_on = False # disable making a copy of object for algo script


    def get_args(self):
        """
        Collects arguments for execution algorithm into a dictionary.

        Args:
            None.

        Returns:
            Dictionary with arguments for execution.
        """

        # Update selection with a current values

        # Collect current values for the execution
        selected_ids = self.get_local_selection()
        selection_level = self.get_selection_level()
        min, max = self.get_limits()
        min_selected = 0

        if self.is_selection_valid(selected_ids, min_selected):
            return {
                'selected_ids': selected_ids,
                'result_name': self.get_result_name(),
                'selection_level': selection_level,
                'Tol_min' : min,
                'Tol_max' : max,
                'Debug'   : self._debug.isChecked()
            }

        return None

    def onDebugStatechanged(self,):
        if self._debug.isChecked():
            print("Debug is checked")
        else:
            print("Debug is unchecked")

    def get_limits(self):
        """
        Returns current values for min/max limits.

        Args:
            None.

        Returns:
            List of limits [min, max].
        """

        return [self._min_widget.SpinBox_DX.value(),
                self._max_widget.SpinBox_DX.value()]


    def set_limits(self, min_value, max_value):
        """
        Sets given values for min/max limits.

        Args:
            None.

        Returns:
            None.
        """

        self._min_widget.SpinBox_DX.setValue(min_value)
        self._max_widget.SpinBox_DX.setValue(max_value)

    def on_limit_changed(self):
        """
        One of the limits was changed.

        Args:
            None.

        Returns:
            None.
        """

        # TODO: Do we need an interactive change here?
        # self.select_subshapes_in_limits()
        print('limit changed')
        min, max = self.get_limits()
        print("{} - {}".format(min, max))


    def on_select_object(self):
        """
        Override parent's method to display sub-shapes info.

        Args:
            None.

        Returns:
            None.
        """

        # Call parent method first
        super().on_select_object()


# For testing run as a module from geomrepairadv parent directory in
# Salome INSTALL, because the dialog needs a generated Ui_BaseDlg class
# that we don't have in the SOURCE.
# Example:
# $ python -m geomrepairadv.reparation_plugin
if __name__ == '__main__':
    app = QApplication(sys.argv)

    dlg = Reparation_plugin(None)
    dlg.show()

    sys.exit(app.exec_())
